#!/usr/bin/env python

# Wiretap SDK tool for creating a video clip.
# The tool can link to media (soft-importing). 
# It also indicates how to write media to the clip.

import getopt
import sys
import os
from libwiretapPythonClientAPI import *

def printUsage():

    print " Usage:"
    print "  -n <parent node ID>" 
    print "  -N <number of frames to create> (default = 1)" 
    print "  -l <directory of media files to which to link>" 
    print "  -l <file of FULL paths to image files (newline separated)>" 
    print "  -t <server-specific node type> (default = '0')" 
    print "  -d <display name> (default = node)" 
    print "  -x <frame width> (default = 720)" 
    print "  -y <frame height> (default = 486)" 
    print "  -c <num channels> (default = 3)" 
    print "  -r <frame rate> (default = 30)" 
    print "  -b <bits per pixel: 24, 30, 48, etc> (default = 24)" 
    print "  -f <format tag: rgb, dpx, sgi, etc> (default = rgb)" 
    print "  -s <scan format: %s, %s, %s, %s, %s>" % (
           WireTapClipFormat.SCAN_FORMAT_FIELD_1_ODD_STR(),
           WireTapClipFormat.SCAN_FORMAT_FIELD_2_ODD_STR(),
           WireTapClipFormat.SCAN_FORMAT_FIELD_1_EVEN_STR(),
           WireTapClipFormat.SCAN_FORMAT_FIELD_2_EVEN_STR(),
           WireTapClipFormat.SCAN_FORMAT_PROGRESSIVE_STR() )
    print "  -p <pixel ratio>" 
    print "  -m <metadata tag>" 
    print "  -M <metadata filename>" 
    print "  -h <host name or IP address> (default = localhost)" 
    print "  -C <colour space> (default = none)\n" 

def main(argv):
 
  # Check the command line options for correctness.
  #
  try:
    opts, args = getopt.getopt(argv, "n:N:l:t:d:x:y:c:r:b:f:s:p:m:M:h:C:")
  except getopt.GetoptError:
    printUsage()
    sys.exit(2)

  # Declare variables (to be set from command line arguments).
  # For hostName, the default "localhost" will work if a Wiretap server 
  # is running on your machine.
  # For nodeType, "NODE" is a type defined by the IFFFS Wiretap Server.  
  #
  hostName = "localhost"
  parentNodeId = ""
  nodeType = "CLIP"
  displayName = "node"
  numFrames = 1
  frameDir = ""

  # Define a clip format. The parameters allow the server to
  # determine the frame buffer size for the specified frame format
  # (in this example, an NTSC raw RGB frame). The buffer size calculated
  # by the server can be obtained after the clip has been created.
  #
  format = WireTapClipFormat( 720, 486, 24, 3, 0, float(30.0), float(1.0),
                            WireTapClipFormat.ScanFormat.SCAN_FORMAT_FIELD_1_ODD,
                            WireTapClipFormat.FORMAT_RGB() )

  # Parse command line and set variables declared earlier.
  # 
  for opt, arg in opts:
    if opt == '-h':
      hostName = arg
    elif opt == '-n':
      parentNodeId = arg
    elif opt == '-d':
      displayName = arg
    elif opt == '-t':
      nodeType = arg
    elif opt == '-N':
      numFrames = int( arg )
    elif opt ==  '-l':
      frameDir = arg 
    elif opt ==  '-x':
      format.setWidth( int( arg ) )
    elif opt == '-y':
      format.setHeight( int( arg ) )
    elif opt == '-c':
      format.setNumChannels( int( arg ) )
    elif opt == '-r':
      format.setFrameRate( float( arg ) )
    elif opt == '-b':
      format.setBitsPerPixel( int( arg ) )
    elif opt == '-f':
      format.setFormatTag( arg )
    elif opt == '-p':
      format.setPixelRatio( float( arg ) )
    elif opt == '-s': 
      f = WireTapClipFormat.strToScanFormat( arg )
      if f == -1:
        printUsage()
        raise Exception( "Unknown clip format specified" )
      format.setScanFormat( f )
    elif opt == '-M':
      try: 
        fileHandle = open( arg, 'r' )
        format.setMetaData( fileHandle.read() )
      except IOError,e: 
        raise Exception( "Cannot read meta data file '%s': %s" % ( arg, e ) )
    elif opt == '-m':
      format.setMetaDataTag( arg )
    elif opt == '-C':
      format.setColourSpace( arg )

  if parentNodeId == "":
    print "Parent Node ID not specified.\n"
    printUsage()
    sys.exit(2)

  # Initialize the Wiretap Client API.
  #
  wiretapClient = WireTapClient()
  if not wiretapClient.init():
    raise Exception( "Unable to initialize WireTap client API" )

  try:
    # Instantiate a server handle
    #
    server = WireTapServerHandle( hostName )

    # SOFT-IMPORTING (the -l option must have been set)
    # See also the block labeled "WRITING FRAMES" below.
    # 
    # Declare a list to store the paths to the frame files, 
    #
    paths = []

    # If the -l option was set and is a directory, 
    # add the files in the specified directory to the paths list
    #
    if os.path.isdir(frameDir): 
      paths = []
      for file in os.listdir( frameDir ):
        paths.append( WireTapStr ( os.path.join( frameDir, file ) ) )

    # If the -l option was set and is a file,
    # read the file to get paths to frame files--skipping the new line separator.
    #
    elif os.path.isfile(frameDir): 
      fileHandle = open( frameDir, 'r' )
      for line in fileHandle.readlines():
        if line != "\n":
          paths.append( WireTapStr ( line.rstrip( '\n' ) ) )

    # Create the clip node.
    #
    parentNode = WireTapNodeHandle( server, parentNodeId )
    node = WireTapNodeHandle()

    if not parentNode.createClipNode( displayName, format, nodeType, node ):  
      raise Exception( "Cannot create clip node: %s" % parentNode.lastError() )

    print "Created clip node '%s'." % node.getNodeId().id() 

    # Link to the media if paths were specified.
    #
    if ( len(paths) > 0 ): 
      if not node.linkToFrames( paths ):
        node.destroyNode()
        raise Exception( "Cannot link to media: %s" % node.lastError() )
      print "Link to %d frames complete "% len( paths )

    # WRITING FRAMES (the -N option must have been set).
    # An alternative to "SOFT-IMPORTING"--see above.

    # If the -N option was set, set the number of frames to be 
    # written to the clip (without actually writing the frames).
    #
    elif ( numFrames > 0 and \
           ( len( format.metaDataTag() ) == 0 or \
             format.metaDataTag() == "none" ) ):
      if not node.setNumFrames( numFrames ): 
        node.destroyNode()
        raise Exception( "Cannot allocate %d frames: %s"% ( numFrames, node.lastError() ) )
      print "Successfully allocated %d frames "% numFrames 

  finally:
    # Must destroy WireTapServerHandle and WireTapNodeHandle before
    # uninitializing the WireTap Client API.
    #
    node = None
    parentNode = None
    server = None

if __name__ == '__main__':
  main(sys.argv[1:])

